;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; Powerup Roulette, by imamelia
;;
;; This sprite rapidly flickers between different powerups, and when the player hits
;; it, he/she receives the powerup specified.
;;
;; Extra bytes: 1
;;
;; Extra byte 1:
;;
;; Bits 0-6: Index to property tables.
;; Bit 7: Affects item memory.
;;
;; Sprite table info:
;;
;; $C2,x = sprite state
;; $1504,x = current behavior setting (index in the cycle)
;; $1510,x = mirror of $7FAB40,x
;; $151C,x = Y position of the sprite when it activates (low byte)
;; $1528,x = primary property value (MainProperties)
;; $1534,x = secondary property value (SpawnNumber)
;; $1540,x = timer for blinking and finishing subroutine activation
;; $1570,x = timer
;; $160E,x = number of values in the cycle
;; $187B,x = Y position of the sprite when it activates (high byte)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

incsrc subroutinedefs.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

!ActivateSFX = $0C1DF9		; sound effect number and sound bank used when the player activates the sprite
!TimeToBlink = $40			; how many frames it will blink after being hit
!TimeTillActivate = $7F		; how many frames will pass until it activates a finishing subroutine (if set to do so)
!YSpeed = $D8			; Y speed of the sprite when moving upward after being hit

; Pointers to the tables of values that will correspond to what the roulette does.
; There is no need to change these, but this table can be expanded up to FF.
BehaviorPtrs:
dw BehaviorTbl00
dw BehaviorTbl01
dw BehaviorTbl02
dw BehaviorTbl03

; This table indicates how many values the roulette will cycle through.
; 01 = 1 value, 02 = 2 values, etc.
; (I would advise against putting 00 or 01 here.)
CyclePeriod:
db $0C,$04,$06,$04

; Each of these tables has twice as many values as the corresponding
; "CyclePeriod" value.  If the roulette cycles through 4 values, the table
; will be 8 bytes long; if the roulette cycles through 6 values, the table
; will be 12 bytes long, etc.  The even bytes are what index to use for
; all the property tables, and the odd bytes are how many frames the
; icon and behavior corresponding to that value will be active.
BehaviorTbl00:
db $00,$18,$01,$18,$02,$18,$03,$18,$04,$18,$05,$18
db $06,$18,$07,$18,$08,$18,$09,$18,$0A,$18,$0B,$18
BehaviorTbl01:
db $00,$08,$01,$08,$02,$08,$03,$08
BehaviorTbl02:
db $00,$08,$01,$08,$02,$08,$03,$08,$04,$08,$05,$08
BehaviorTbl03:
db $05,$10,$06,$0C,$07,$08,$08,$04

; Each of the following tables corresponds to values in the behavior tables.
; For instance, a 00 in a behavior table will use the first value from each of
; these tables, a 01 will use the second, and so on.

; Bits 0-1: High byte of the sprite number, if custom.
; Bits 2-3: What to do after spawning the sprite.
;	00 - nothing
;	01 - fade to overworld and activate normal exit
;	10 - fade to overworld and activate secret exit
;	11 - teleport to the level set by the screen exit of the screen the sprite is in
; Bits 4-5: What to do when spawning.
;	00 - spawn normal sprite
;	01 - spawn custom sprite
;	10 - put an item in the item box
;	11 - activate a subroutine
; Bits 6-7: Unused.
MainProperties:
db $00,$00,$00,$00,$00,$30,$30,$30	; values 00-07
db $30,$30,$11,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; spawned sprite number/sprite number low byte/item box value/subroutine number
SpawnNumber:
db $74,$75,$76,$77,$78,$00,$01,$02	; values 00-07
db $03,$04,$00,$04,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; spawned sprite status
SpawnStatus:
db $08,$08,$08,$08,$08,$08,$08,$08	; values 00-07
db $08,$08,$08,$09,$08,$08,$08,$08	; values 08-0F
;db $08,$08,$08,$08,$08,$08,$08,$08	; values 10-17

; first extra byte of the spawned sprite
SpawnEB1:
db $00,$00,$00,$00,$00,$00,$00,$00	; values 00-07
db $00,$00,$08,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; second extra byte of the spawned sprite
SpawnEB2:
db $00,$00,$00,$00,$00,$00,$00,$00	; values 00-07
db $00,$00,$00,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; third extra byte of the spawned sprite
SpawnEB3:
db $00,$00,$00,$00,$00,$00,$00,$00	; values 00-07
db $00,$00,$00,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; fourth extra byte of the spawned sprite
SpawnEB4:
db $00,$00,$00,$00,$00,$00,$00,$00	; values 00-07
db $00,$00,$00,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; X speed of the spawned sprite
SpawnXSpeed:
db $00,$00,$00,$00,$00,$00,$00,$00	; values 00-07
db $00,$00,$00,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; Y speed of the spawned sprite
SpawnYSpeed:
db $00,$00,$00,$00,$00,$00,$00,$00	; values 00-07
db $00,$00,$00,$00,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; sprite tile number to show
Tilemap:
db $24,$26,$48,$0E,$24,$E8,$E8,$E8	; values 00-07
db $E8,$C0,$88,$8C,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

; properties of the tile to show (YX--CCCT)
TileProps:
db $08,$0A,$04,$04,$0A,$04,$08,$0A	; values 00-07
db $06,$06,$0B,$0A,$00,$00,$00,$00	; values 08-0F
;db $00,$00,$00,$00,$00,$00,$00,$00	; values 10-17

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Init:

INC $1602,x			;
LDA $D8,x			;
SEC					;
SBC #$40				;
STA $151C,x			;
LDA $14D4,x			;
SBC #$00				;
STA $187B,x			;
LDA $7FAB40,x		;
PHP					;
AND #$7F			;
STA $1510,x			;
TAY					;
LDA CyclePeriod,y		;
STA $160E,x			;
PLP					;
BPL .EndInit			; if bit 7 is set, it will affect item memory

JSR GetItemMemoryBit	; check the item memory bits
BEQ .EndInit			; if the relevant item memory bit has not been set, then nothing happens

STZ $14C8,x			; if this sprite has already been collected, erase it

.EndInit				;
RTL					;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Main:
JSR PowerupRouletteMain
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PowerupRouletteMain:

LDA $1510,x			;
REP #$30				;
AND #$00FF			;
ASL					;
TAY					;
LDA BehaviorPtrs,y		;
STA $08				; $08-$09 = pointer to behavior table
SEP #$30				;
LDA $1504,x			;
ASL					;
TAY					;
REP #$20				;
LDA ($08),y			;
STA $0A				; $0A = property byte (index to spawn tables and the like)
STA $14B0			;
SEP #$20				; $0B = frames to show the current icon

LDA $1602,x			;
BEQ .NoGFX			;
JSR PowerupRouletteGFX	;
.NoGFX				;

LDA $14C8,x
CMP #$08
BNE .Return
LDA $9D
BNE .Return

JSL !SubOffscreenX0

LDA $C2,x			;
BNE .NoChange		;
INC $1570,x			; increment the frame counter
LDA $1570,x			;
CMP $14B1			; if the frames have reached maximum...
BCC .NoChange		; change the icon
STZ $1570,x			; reset the frame counter
INC $1504,x			; increment the cycle index
LDA $1504,x			;
CMP $160E,x			; if the cycle has ended...
BCC .NoChange		; restart it
STZ $1504,x			;
.NoChange			;

LDA $C2,x
JSL $8086DF

dw State00	; 00 - flickering between powerups
dw State01	; 01 - moving up
dw State02	; 02 - spawned something
dw State03	; 03 - activating any necessary code after spawning something

.Return
RTS


;------------------------------------------------
; sprite state 00
;------------------------------------------------

State00:

JSL $81A7DC				; check for player/sprite contact
BCC .Return				; return if there is none

INC $C2,x				; increment the sprite state
LDA.b #!ActivateSFX>>16	;
STA.w !ActivateSFX&$FFFF	; play a sound effect
LDA #!YSpeed				;
STA $AA,x				;

.Return					;
RTS						;

;------------------------------------------------
; sprite state 01
;------------------------------------------------

State01:

LDA $1540,x				;
BNE .Blinking				;
JSL $81801A				; update sprite Y position

LDA $151C,x				; check how far up the sprite has moved
STA $00					;
LDA $187B,x				;
STA $01					;
LDA $14D4,x				;
XBA						;
LDA $D8,x				;
REP #$20					;
CMP $00					; if it has reached the critical point...
SEP #$20					;
BCC .SetBlinkTimer			; make it start blinking
.Return					;
RTS						;

.Blinking					;
LDA $1540,x				;
CMP #$01				; if it is down to 01, go to the next sprite state
BEQ .IncState				;
LSR #3					;
AND #$03				;
STA $1602,x				;
RTS						;

.SetBlinkTimer				;
LDA #!TimeToBlink			;
STA $1540,x				;
RTS						;

.IncState					;
INC $C2,x				;
STZ $1602,x				;
RTS						;

;------------------------------------------------
; sprite state 02
;------------------------------------------------

State02:

LDA $7FAB40,x		;
BPL .NoSetItemMemory	; if the sprite affects item memory...
JSR SetItemMemoryBit	; set the relevant bit
.NoSetItemMemory		;

LDY $14B0			;
LDA MainProperties,y	;
STA $1528,x			;
LDA SpawnNumber,y	;
STA $1534,x			;

PEA.w .Return-$01		;
LDA $1528,x			;
LSR #4				;
AND #$03			;
JSL $8086DF			;

dw SpawnSprite		;
dw SpawnSprite		;
dw ItemInBox			;
dw ActivateSub		;

.Return				;
INC $C2,x			;
LDA #!TimeTillActivate	;
STA $1540,x			;
RTS					;

ItemInBox:

LDA $1534,x			;
STA $0DC2			;
RTS					;

ActivateSub:

LDA $1534,x			;
JSL $8086DF			;

dw Give1Coin			;
dw Give3Coins			;
dw Give5Coins			;
dw Give10Coins		;
dw DisplayMsg1		;
dw CustSub05			;
dw CustSub06			;
dw CustSub07			;
dw CustSub08			;
;dw CustSub09		;
;...
;dw CustSubFF			;

CustSub05:
CustSub06:
CustSub07:
CustSub08:
RTS

Give1Coin:			;
LDA #$01			;
BRA GiveCoins			;

Give3Coins:			;
LDA #$03			;
BRA GiveCoins			;

Give5Coins:			;
LDA #$05			;
BRA GiveCoins			;

Give10Coins:			;
LDA #$0A			;

GiveCoins:			;
JSL $85B329			;
RTS					;

DisplayMsg1:			;
INC $1426			;
RTS					;

;------------------------------------------------
; sprite state 03
;------------------------------------------------

State03:

LDA $1540,x			; if the activation timer is still set...
BNE .Return			; return

LDA $1528,x			;
LSR #2				;
AND #$03			;
JSL $8086DF			;

dw .Erase				;
dw .EndLevel			;
dw .EndLevel			;
dw .Teleport			;

.Erase				;
STZ $14C8,x			;
.Return				;
RTS					;

.EndLevel

LDA $1528,x			;
LSR #2				;
DEC					;
AND #$01			;
STA $0DD5			;
STA $13CE			;
INC $1DE9			;
LDA #$0B				;
STA $0100			;
RTS					;

.Teleport

LDA $5B				;
LSR					;
BCS .Vertical			;
LDY $14E0,x			;
LDX $95				;
BRA .Continue			;
.Vertical				;
LDY $14D4,x			;
LDX $97				;
.Continue				;
LDA $19B8,y			;
STA $19B8,x			;
LDA $19D8,y			;
ORA #$04			;
STA $19D8,x			;
LDX $15E9			;
LDA #$06			;
STA $71				;
STZ $88				;
STZ $89				;
RTS					;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PowerupRouletteGFX:

JSL !GetDrawInfo

LDA $00
STA $0300,y

LDA $01
STA $0301,y

LDX $0A
LDA.w Tilemap,x
STA $0302,y

LDA.w TileProps,x
ORA $64
STA $0303,y

LDX $15E9
LDY #$02
LDA #$00
JSL $81B7B3
RTS


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; subroutines
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;------------------------------------------------
; spawning routines
;------------------------------------------------

SpawnSprite:

JSL $82A9E4
BPL .Continue
RTS

.Continue

LDA $1528,x
STA $08
LDA $1534,x
STA $09

LDA $E4,x
STA $00E4,y
LDA $14E0,x
STA $14E0,y
LDA $D8,x
STA $00D8,y
LDA $14D4,x
STA $14D4,y

LDX $0A
LDA.w SpawnStatus,x
STA $14C8,y
TYX
LDY $0A
LDA $08
AND #$10
BNE .SpawnCustom

.SpawnNormal

LDA $09
STA $9E,x
JSL $87F7D2
BRA .Shared

.SpawnCustom

LDA $09
STA $7FAB9E,x
LDA $08
AND #$03
ORA #$80
STA $7FAB10,x
JSL $81830B
LDA SpawnEB1,y
STA $7FAB40,x
LDA SpawnEB2,y
STA $7FAB4C,x
LDA SpawnEB3,y
STA $7FAB58,x
LDA SpawnEB4,y
STA $7FAB64,x

.Shared

LDA SpawnXSpeed,y
STA $B6,x
LDA SpawnYSpeed,y
STA $AA,x
LDX $15E9
RTS

;------------------------------------------------
; smoke routine
;------------------------------------------------

SubSmoke:

PHY
LDY #$03
.FindFree
LDA $17C0,y
BEQ .FoundOne
DEY
BPL .FindFree
PLY
RTS

.FoundOne

LDA #$01
STA $17C0,y
LDA $D8,x
STA $17C4,y
LDA $E4,x
STA $17C8,y
LDA #$1B
STA $17CC,y

PLY
RTS

;------------------------------------------------
; item memory subroutines (for sprites; adapted from the object version at $0DA8DC)
;------------------------------------------------

GetItemMemoryBit:

PHX					;
PHY					;

JSR ItemMemoryIndexRt	;

LDA ($08),y			; item memory pointer
AND $818000,x		; check a particular bit
STA $0F				;

PLY					;
PLX					;
LDA $0F				;
RTS					;


SetItemMemoryBit:

PHX					;
PHY					;

JSR ItemMemoryIndexRt	;

LDA ($08),y			; item memory pointer
ORA $818000,x		; set a particular bit
STA ($08),y			;

PLY					;
PLX					;
RTS					;


ItemMemoryIndexRt:

LDA $5B				;
LSR					;
BCS .VertLevelSetup		;

LDA $E4,x			;
STA $0A				;
LDA $D8,x			;
STA $0B				;
LDA $14E0,x			;
STA $0C				;
LDA $14D4,x			;
STA $0D				;
BRA .Continue			;

.VertLevelSetup		;
LDA $D8,x			;
STA $0A				;
LDA $E4,x			;
STA $0B				;
LDA $14D4,x			;
STA $0C				;
LDA $14E0,x			;
STA $0D				;

.Continue				;

LDX $13BE			; item memory setting
LDA #$F8				; base address low byte
CLC					;
ADC $8DA8AE,x		; plus offset
STA $08				;
LDA #$19			; base address high byte
ADC $8DA8B1,x		; plus offset
STA $09				; forms a 16-bit pointer

LDA $0C				; screen number (high byte of X position, or Y in vertical levels)
ASL #2				;
STA $0E				;
LDA $0D				;
BEQ .UpperSubscreen	; if the sprite is on the lower subscreen...
LDA $0E				;
ORA #$02			;
STA $0E				;
.UpperSubscreen		;
LDA $0A				;
AND #$80			; if the sprite is on the left half of the subscreen...
BEQ .LeftHalf			;
LDA $0E				;
ORA #$01			;
STA $0E				;
.LeftHalf				;
LDA $0A				;
LSR #4				;
AND #$07			;
TAX					; get the bit index into the table
LDY $0E				; get the byte index
RTS					;


dl Init,Main











